/**********************************************************************************************
***********************************************************************************************
Macro Name: UNI_PHREG
Created Date/Author: Feb. 2012/Yuan Liu
Last Update Date/Person: June 10, 2014/Dana Nickleach
Current Version: V23
Working Environment: SAS 9.3 English version

Contact: Dr. Yuan Liu yliu31@emory.edu

Purpose:  To conduct univariate survival analysis for each variable in the dataset. The hazard 
ratio with 95% CI is presented along with the log rank test p-value.  For categorical 
variables, the reference group will be shown along with the number of observation in each 
category.  The proportional hazard assumption can optionally be checked.

Notes: 1) The order of variables in the summary table is the same as the input order. For the 
best results, you may want to put the demographic variables together and also clinical 
characteristics variables together; 2) if the sample size is too big, the 
proportional hazard assumption check option may not work well.  In that case, please disable 
the option.

Parameters: 

DATASET        The name of the data set to be analyzed.
EVENT          The variable name for time to event outcome.
START          Instead of a single failure-time variable (EVENT) a pair of failure time 
               variables can be specified.  This should be the start point of the interval for
               which the subject is at risk.  The label of this variable will be used in the
               table header.
STOP           Instead of a single failure-time variable (EVENT) a pair of failure time 
               variables can be specified.  This should be the end point of the interval for
               which the subject is at risk.  
CENSOR         The variable name for censoring indicator. It is required that 1 is used for the 
               event and 0 for censored cases. 
CLIST          List of categorical variables, separated by empty space. Each variable name must 
               be less than 31 characters long.
CREFLIST       List of the categorical variables with reference levels specified separated by * 
               (optional).    
NLIST          List of numerical variables, separated by empty space.
DOC            Set to T to create a RTF file containing the output or F to suppress creation of 
               the RTF file (optional).  The default value is T.
OUTPATH        Path for output table to be stored.
FNAME          File name for output table.
PHA            Value of F or T to indicate whether to run the proportional hazard assumption 
               check.
TYPE3          Value of F or T to indicate whether to print the type 3 Wald p-value in the 
               table (optional).  The default value is F.
LOGRANK        Value of F or T to indicate whether to print the log-rank p-value in the table 
               (optional).  The default value is T.  
FIRTH          Set to T to use Firths penalized maximum likelihood estimation to reduce bias
               in the parameter estimates.  The default value is F.
PL             Set to T to use the profile penalized log likelihood method for HR confidence 
               intervals, HR p-values, and type III p-values instead of the Wald method.  This 
               can only be used in conjunction with FIRTH=T.  The COXPHFIRTH module, R, and the 
               R package coxphf are required.  The RLANG option must also be in use in SAS.  
               The default value is F.  
WEIGHT         The name of the variable to include in the weight statement in PHREG (optional).
               Note that weights will not be normalized.  All N's reported will be 
               non-weighted.
ID             ID variable for identifying observations to be used in the ID statement in
               PHREG (optional).  If observations have more than one record then the N's 
               reported are for the unique number of ID's not records.  One ID is assumed to
               have the same covariates across all records.This option is used for counting process
			   data format along with START and STOP options.
MATCH_ID	   Specify the Match ID in the matched sample. This should be used along with COVSAGG = T.	
COVSAGG        Set to T to use the COVS(AGGREGATE) option in PHREG (optional). The default 
               value is F. If ID or MATCH_ID, then set to T.
ORIENTATION    Value of PORTRAIT or LANDSCAPE to indicate the paper layout of the report.
DEBUG          Set to T if running in debug mode (optional).  Work datasets will not be deleted 
               in debug mode.  This is useful if you are editing the code or want to further 
               manipulate the resulting data sets.  The default value is F.

***********************************************************************************************
For more details, please see the related documentation
**********************************************************************************************/

%MACRO UNI_PHREG(DATASET=, EVENT=, CENSOR=, START=, STOP=, CLIST =, CREFLIST=, NLIST=, DOC=T, 
   OUTPATH=, FNAME=, PHA=F, LOGRANK=T, TYPE3=F, FIRTH=F, PL=F, WEIGHT=, ID=, MATCH_ID = , COVSAGG=F, 
   ORIENTATION=PORTRAIT, DEBUG=F); 

   %local I __MACRO_ERR WORK_SETS CHECK REFVAR EVENTVV LOGRANKP C assump cref format;

   /* Capitalize */
   %let pha = %UPCASE(&pha);
   %let debug = %UPCASE(&debug);
   %let logrank = %UPCASE(&logrank);
   %let doc = %UPCASE(&doc);
   %let covsagg = %UPCASE(&covsagg);
   %let type3 = %UPCASE(&type3);
   %let firth = %UPCASE(&firth);
   %let pl = %UPCASE(&pl);

   %IF &PHA = TRUE %then %let PHA = T;

   /* Count number of variables */
   %if &clist = %STR() %then %do;
      %let clist_cnt = 0;
   %end;
   %else %do;
      %let clist_cnt = %sysfunc(countw(&CLIST));
   %end;

   /* Initialize error flag */
   %let __Macro_Err = 0;

   /* Check for categorical variable names that are too long (needed for proc freq) */
   %DO i = 1 %to &clist_cnt; 
      %IF %LENGTH(%SCAN(&CLIST, &i)) > 30 %then %do;
         %put ERROR: Variable name %SCAN(&CLIST, &i) must be less than 31 characters.; 
           %let __Macro_Err=1;
       %END;
   %END;

   /* Make sure that each categorical variable has at least two non-missing values */
   %do i = 1 %to &clist_cnt;
      PROC SQL noprint;
         select count(distinct %SCAN(&clist, &i)) into :check
         from &dataset
         where MISSING(%SCAN(&clist, &i)) = 0;
      QUIT;

       %if &check <= 1 %then %do;
          %put ERROR: The variable %SCAN(&clist, &i) has less than two non-missing levels.  Please remove from CLIST.; 
           %let __Macro_Err=1;
       %end;
   %end;

   /* Can't use weights with the log-rank test */
   %if &weight ~= %STR() and &logrank = T %then %do;
      %put ERROR: Weights cannot be used with the log-rank test.  Please set LOGRANk=F.; 
      %let __Macro_Err=1;
   %end;

   /* Can't use weights with the PH test */
   %if &weight ~= %STR() and &pha = T %then %do;
      %put ERROR: The PH assumption cannot be tested when weights are used.  Please set PHA=F.; 
      %let __Macro_Err=1;
   %end;

   /* Should not specify both EVENT and START/STOP */
   %if &event ~= %STR() and &start ~= %STR() and &stop ~= %STR() %then %do;
      %put ERROR: Either EVENT and START/STOP, but not both should be specified.; 
      %let __Macro_Err=1;
   %end;

   /* Can't use log-rank test with start/stop format */
   %if &start ~= %STR() and &stop ~= %STR() and &logrank=T %then %do;
      %put ERROR: Log-rank p-values cannot be reported when a START/STOP time is specified.  Please set LOGRANK=F.; 
      %let __Macro_Err=1;
   %end;

   /* Can't use PL method with start/stop format */
   %if (&start ~= %STR() OR &stop ~= %STR()) and &pl=T %then %do;
      %put ERROR: Penalized likelihood cannot be used when a START/STOP time is specified.  Please set PL=F.; 
      %let __Macro_Err=1;
   %end;

   /* Can't use PL method with ID */
   %if &id ~= %STR() and &pl=T %then %do;
      %put ERROR: Penalized likelihood cannot be used in conjunction with ID.  Please set PL=F.; 
      %let __Macro_Err=1;
   %end;

   /* Can't use PL method with ID */
   %if &COVSAGG = T and &pl=T %then %do;
      %put ERROR: Penalized likelihood cannot be used in conjunction with COVSAGG.  Please set PL=F.; 
      %let __Macro_Err=1;
   %end;

   /* Can't use PL method with PHA */
   %if &pha = T and &pl=T %then %do;
      %put ERROR: Penalized likelihood cannot be used when a proportional hazards check is requested.  Please set PL=F or PHA=F.; 
      %let __Macro_Err=1;
   %end;

   /* Can't use PL method with weight */
   %if &weight ~= %STR() and &pl=T %then %do;
      %put ERROR: Penalized likelihood cannot be used with weights.  Please set PL=F.; 
      %let __Macro_Err=1;
   %end;

   /* Can only use PL with FIRTH */
   %if &pl = T and &firth ~= T %then %do;
      %put ERROR: Penalized likelihood can only be used with Firth.  Please set PL=F or FIRTH=T.; 
      %let __Macro_Err=1;
   %end;

   %if &__Macro_Err. %then %do;
      data _null_;
         abort 3;
      run;
   %end;

   /* Get list of data sets in work library to avoid deletion later */
   ODS OUTPUT Members(nowarn)=_DataSetList;
   PROC DATASETS lib=work memtype=data;
   QUIT;

   /* If there are data sets in the work library */
   %if %sysfunc(exist(_DataSetList)) %then %do;
      PROC SQL noprint;
         select Name
         into :work_sets separated by ' '
         from _DataSetList;
      quit;
   %end;
   %else %do;
      %let work_sets =;
   %end;

   /* Save current options */
   PROC OPTSAVE out=_options;
   RUN;

   /* If penalized likelihood is used then modeling will have to be done in R */
   /* Export data to CSV */
   %if &PL = T %then %do;

      /* Get exact variable name (upper/lower case specific) for use in R */
      DATA _NULL_;
         set &dataset;
         CALL SYMPUT('event',VNAME(&EVENT));
         CALL SYMPUT('censor',VNAME(&censor));
      RUN; 

   %end;

   *CHARACTER VARIABLES ;

   %IF &CLIST NE  %THEN %DO; 
      %LET C = 1; 

      %DO %UNTIL (%SCAN(&CLIST, &C) =   ); 
         %LET CVAR = %SCAN(&CLIST, &C); 
         %LET REFVAR = %SCAN(&CREFLIST,&C,*); 

         /* If no reference categories were provided then use the default */
         %if %sysevalf(%superq(REFVAR)=,boolean) %then %do;
            %let refvar = &cvar;
         %end;

         /* For datasets where records does not represent the number of observations */
         /* Return to observation level for frequency calculations */
         %if &id ~= %STR() %then %do;
            PROC SORT DATA =&dataset out = _uni nodupkey;
               by &id;
            RUN;
         %end;
         %else %do;
            DATA _uni;
               set &dataset;
            RUN;
         %end;

         ODS OUTPUT 'One-Way Frequencies' = _FREQ; 
         PROC FREQ DATA=_uni ORDER=internal; 
            TABLE &CVAR;
            %if &event ~= %STR() %then %do;
               WHERE &EVENT ~= . and &censor ~= .;
            %end;
            %else %do;
               WHERE &START ~= . and &stop ~= . and &censor ~= .;
            %end;
         RUN;  

         DATA _FREQC; 
            SET _FREQ ; 
            LENGTH Covariate $256. Level $256.;
            Covariate = label(&CVAR); 
            level = LEFT(F_&CVAR); 
            N =  Frequency;
            KEEP Covariate level Frequency N; 
         RUN;

         %if &logrank = T %then %do; 
            /* Calculate log-rank test */
            ODS OUTPUT HomTests=_score;
            PROC LIFETEST DATA = &dataset NOTABLE;
               time &EVENT*&CENSOR(0);
               strata &cvar;
            RUN;

            data _score;
               set _score;
               format ProbChiSq;
            run;

            /* Get log-rank p-value */
            proc sql noprint; 
               select ProbChiSq into: logrankp 
               from _score where test ="Log-Rank";
            quit;
         %end;
         %else %let logrankp=.;

         /* Run this even when penalized likelihood is used in order to get likelihood ratio 
         test because it is difficult to pull from R */
         ODS OUTPUT %IF &PHA = T %then %do; "Supremum Test for Proportional Hazards Assumption" = _phac %end;
               "Maximum Likelihood Estimates of Model Parameters" = _mlec 
               "Number of observations" = _nobs Type3 = _typ3  GlobalTests=_global;
         proc phreg data=&dataset %if &covsagg=T %then %do; COVS(AGGREGATE) %end;;
            class &refvar/order=internal;
            %if &event ~= %STR() %then %do;
               model &EVENT*&CENSOR(0) = &CVAR/rl %if &firth = T %then %do; firth %end;;
            %end;
            %else %do;
               model (&start,&stop)*&CENSOR(0) = &CVAR/rl %if &firth = T %then %do; firth %end;;
            %end;
            %if &PHA = T %then %do; 
               assess ph/resample seed=1;
            %end;
            %if &weight ~= %STR() %then %do;
               weight &weight;
            %end;
            %if &id ~= %STR() %then %do;
               id &id;
			%end;
			%if &match_id ~= %STR() %then %do;
               id &match_id;
			%end;
         Run;

         data _mlec;
            set _mlec;

            /* Preserve order */
            order2 = _n_;
            format ProbChiSq;
         run;

         %IF &PHA = T %THEN %DO;
              
            /* Merge on PHA check */
            DATA _freqc;
               if _n_ = 1 then set _phac;
               set _freqc;
               format pvalue;
            RUN;

         %end;

         PROC SORT DATA = _freqc;
            by level;
         RUN;
         PROC SORT DATA =  _mlec (rename=(ClassVal0=level));
            by level;
         RUN;

         data _freqcc; 
            merge _freqc _mlec;
            by level;

            /* Make sure reference level comes last */
            if order2 = . then order2 = 999;
         run;

         %if &pl = T %then %do;
            /* Get reference level */
            PROC SQL noprint;
               select level into :cref
               from _freqcc
               where Estimate = .;
            QUIT;

            /* Get exact variable name (upper/lower case specific) for use in R */
            DATA _NULL_;
               set &dataset;
               CALL SYMPUT('cvar',VNAME(&cvar));
               CALL SYMPUT('format',VFORMAT(&cvar));
            RUN; 

            /* Convert categorical variables to character using format */
            /* This will work whether or not the original variable is numeric or character */
            PROC SQL;
               create table _temp as
               select &event, &censor, LEFT(put(&cvar,&format)) as &cvar 
               from &dataset;
            quit;

            DATA _temp;
               set _temp;
               /* Eliminate missing values to prevent problems in R */
               if &cvar in (' ','.') then delete;
            RUN;

            /* Call module to calculate penalized likelihood using R */
            /* Data set _firth will contain the results */
            proc iml;
               load module=coxphfirth;
               call coxphfirth("_temp","&event","&censor","&cvar","&cref");
            quit;

            /* Merge to resulting data sets */
            /* These should be in corresponding order (no merging  key) */
            DATA _firth;
               merge _firthnames _firth;
               /* Create level variable */
               length level $256;
               level = SUBSTR(var,%length(&cvar)+1);
            RUN;

            /* Merge results onto summary file */
            PROC SORT DATA = _firth;
               by level;
            RUN;
            DATA _freqcc;
               merge _freqcc _firth;
               by level;
               /* Use estimates from R */
               HazardRatio = HR;
               HRLowerCL = LCL;
               HRUpperCL = UCL;
               ProbChiSq = pval;
               DROP hr lcl ucl pval nobs;
            RUN;
         %end;

         /* Return to order */
         PROC SORT DATA = _freqcc;
            by order2; 
         RUN;

         data _freq&C; 
            /* Merge on type3 p-value */
            /* If PL is used then use likelihood ratio test */
            %if &pl = T %then %do;
               if _n_ = 1 then set _global (where=(test='Likelihood Ratio') 
                  RENAME=(ProbChiSq=p_type3) KEEP=test ProbChiSq);
            %end;
            %else %do;
               if _n_ = 1 then set _typ3 (RENAME=(ProbChiSq=p_type3) KEEP=ProbChiSq);
            %end;
            set _freqcc;
            logrankp= &logrankp;
            /* Preserve order */
            order2 = &c;
            keep order2 Covariate Level Frequency logrankp HazardRatio HRLowerCL HRUpperCL 
               ProbChiSq p_type3 %IF &PHA = T %then %do; pvalue %end;;
         run;
               
         %LET C = %EVAL(&C+1);
      %END; 
      %let C= %eval(&C - 1) ;

      Data _freq_all; 
         set _FREQ1 - _FREQ&C;
           order = 1;
      run;
   %END;

   *NUMERIC VARIABLES ;
   %IF &NLIST NE  %THEN %DO; 
          
      %LET N = 1; 

      %DO %UNTIL (%SCAN(&NLIST, &N) =   ); 
         %LET NVAR = %SCAN(&NLIST, &N); 

         /* If penalized likelihood is not used */
         %if &pl = F %then %do;

            ODS OUTPUT 
             %IF &PHA = T %then %do; "Supremum Test for Proportional Hazards Assumption" = _phac %end;
             "Maximum Likelihood Estimates of Model Parameters" = _mlec 
             "Number of observations" = _nobs;
            proc phreg data=&DATASET %if &covsagg=T %then %do; COVS(AGGREGATE) %end;;
               %if &event ~= %STR() %then %do;
                  model &EVENT*&CENSOR(0) = &NVAR/rl %if &firth = T %then %do; firth %end;;
               %end;
               %else %do;
                  model (&start,&stop)*&CENSOR(0) = &NVAR/rl %if &firth = T %then %do; firth %end;;
               %end;
               %IF &PHA = T %then %do; assess ph/resample seed=1; %end;
               %if &weight ~= %STR() %then %do;
                  weight &weight;
               %end;
               %if &id ~= %STR() %then %do;
                  id &id;
               %end;
			   	%if &match_id ~= %STR() %then %do;
               id &match_id;
			   %end;

            run;

            data _mlec;
               set _mlec; 
               format  ProbChiSq;
            run;

            %IF &PHA = T %then %do;
               data _phac;
                  set _phac; 
                  format pvalue;
               run;
            %end;

            proc sql noprint;
               select HazardRatio into: hr from _mlec;
               select HRLowerCL into: hr_lb from _mlec;
               select HRUpperCL into: hr_ub from _mlec;
               select ProbChiSq into: hrp from _mlec;
               %IF &PHA = T %then %do;
                  select pvalue into: assump from _phac;
               %end;
               %if &id = %STR() %then %do;
                  select SumFreqsUsed into: N_obs from _nobs;
               %end;
               %else %do;
                  select count(distinct &id) into :N_obs 
                  from &dataset
                  %if &event ~= %STR() %then %do;
                     WHERE &EVENT ~= . and &censor ~= . and &nvar ~= .;
                  %end;
                  %else %do;
                     WHERE &START ~= . and &stop ~= . and &censor ~= . and &nvar ~= .;
                  %end; 
               %end;
            quit;

         %end;
         /* If penalized likelihood is used */
         %else %do;
            /* Get exact variable name (upper/lower case specific) for use in R */
            DATA _NULL_;
               set &dataset;
               CALL SYMPUT('nvar',VNAME(&nvar));
            RUN; 

            /* Call module to calculate penalized likelihood using R */
            /* Data set _firth will contain the results */
            proc iml;
               load module=coxphfirth;
               call coxphfirth("&dataset","&event","&censor","&nvar","NA");
            quit;

            /* Get results into macro variables */
            proc sql noprint;
               select hr, LCL, UCL, pval, nobs
               into :hr, :hr_lb,:hr_ub, :hrp, :N_obs
               from _firth;
            quit;


         %end;

         data _NULL_; 
            set &dataset; 
            call symput('VV', put(label(&NVAR),$256.));
         run;

         data _summary&N;
            length covariate $256. Level $71.;
            covariate = "&VV";
            level = ' ';
            Frequency = &N_obs;
            /* Don't report log-rank p-value for numeric variables */
            logrankp =.;
            HazardRatio = &hr;
            HRLowerCL = &hr_lb;
            HRUpperCL = &hr_ub;
            ProbChiSq = &hrp;
            /* Type 3 p-value will be the same as the HR p-value */
            p_type3 = probchisq;
            %IF &PHA = T %THEN %DO; 
               pvalue = &assump; 
            %end;
            /* Preserve order */
            order2 = &n;

            keep order2 Covariate level Frequency logrankp HazardRatio HRLowerCL HRUpperCL 
               ProbChiSq p_type3 %IF &PHA = T %THEN %DO; pvalue %end;;
         run;

         %LET N = %EVAL(&N+1);
      %END; 

      %LET N = %EVAL(&N-1);

      DATA _summary_all; 
         SET _summary1-_summary&N; 
         order = 2;
      RUN; 
   %END;

   /* Combine categorical and numerical covariate results */
   DATA _report; 
      set %IF &CLIST NE %STR() %then %do; _freq_all %end;
          %if &NLIST NE %STR() %then %do; _summary_all %end;;
      /* HR and 95% CI */
      if HazardRatio ~= . then HR = TRIM(LEFT(PUT(HazardRatio,8.2))) || " (" || 
         TRIM(LEFT(PUT(HRLowerCL,8.2))) || "-" || 
         TRIM(LEFT(PUT(HRUpperCL,8.2))) || ")";
      else HR = '-';
   run; 

   *---- table template -----;  
   ODS PATH WORK.TEMPLAT(UPDATE)
   SASUSR.TEMPLAT(UPDATE) SASHELP.TMPLMST(READ);

   PROC TEMPLATE;
   DEFINE STYLE STYLES.TABLES;
   NOTES "MY TABLE STYLE"; 
   PARENT=STYLES.MINIMAL;

     STYLE SYSTEMTITLE /FONT_SIZE = 12pt     FONT_FACE = "TIMES NEW ROMAN";

     STYLE HEADER /
           FONT_FACE = "TIMES NEW ROMAN"
            CELLPADDING=8
            JUST=C
            VJUST=C
            FONT_SIZE = 10pt
           FONT_WEIGHT = BOLD; 

     STYLE TABLE /
            FRAME=HSIDES            /* outside borders: void, box, above/below, vsides/hsides, lhs/rhs */
           RULES=GROUP              /* internal borders: none, all, cols, rows, groups */
           CELLPADDING=6            /* the space between table cell contents and the cell border */
            CELLSPACING=6           /* the space between table cells, allows background to show */
            JUST=C
            FONT_SIZE = 10pt
           BORDERWIDTH = 0.5pt;  /* the width of the borders and rules */

     STYLE DATAEMPHASIS /
           FONT_FACE = "TIMES NEW ROMAN"
           FONT_SIZE = 10pt
           FONT_WEIGHT = BOLD;

     STYLE DATA /
           FONT_FACE = "TIMES NEW ROMAN" 
           FONT_SIZE = 10pt;

     STYLE SYSTEMFOOTER /FONT_SIZE = 9pt FONT_FACE = "TIMES NEW ROMAN" JUST=C;
   END;

   RUN; 

   **** PRINT THE TABLE ************;
   data _NULL_; 
      set &dataset; 
      %if &event ~= %STR() %then %do;
         call symput('eventvv', put(label(&EVENT),$256.));
      %end;
      %else %do;
         call symput('eventvv', put(label(&start),$256.));
      %end;
   run;

   OPTIONS ORIENTATION=&ORIENTATION MISSING = "-" NODATE;
   %if &doc = T %then %do;
      ODS RTF STYLE=TABLES file= "&OUTPATH.&FNAME &SYSDATE..DOC"; 
   %end;
      PROC REPORT DATA=_report HEADLINE HEADSKIP CENTER SPANROWS LS=256 
          STYLE(REPORT)={JUST=CENTER} SPLIT='~' nowd; 
         COLUMNS order order2 Covariate  Level frequency  
               ("&eventvv" '------------------------------------------'( HR
                ProbChiSq 
               %IF &PHA = T %then %do; pvalue %end; 
               %if &type3 = T %then %do; p_type3 %end;
               %if &logrank = T %then %do; logrankp %end;)); 
         DEFINE order/order order=internal noprint;
         DEFINE order2/order order=internal noprint;
         DEFINE Covariate/ order order=data  "Covariate"  STYLE(COLUMN) = {JUST = L}; 
         DEFINE Level/ DISPLAY   "Level"     STYLE(COLUMN) = {JUST =L}; 
         DEFINE frequency/ DISPLAY   "N"     STYLE(COLUMN) = {JUST = C};                
         DEFINE HR/DISPLAY "Hazard Ratio (95% CI)" STYLE(COLUMN) = {JUST = C CellWidth=15%};
         DEFINE ProbChiSq/DISPLAY "HR P-value" STYLE(COLUMN) = {JUST = C CellWidth=8%}  FORMAT=PVALUE5.3; 
         %IF &PHA = T %then %do;
            DEFINE pvalue/order  "Assumption P-value"  STYLE(COLUMN) = {JUST = C} FORMAT=PVALUE5.3; 
            COMPUTE pvalue; 
               IF pvalue <0.05 THEN CALL DEFINE("pvalue", "STYLE", "STYLE=[FONT_WEIGHT=BOLD]"); 
            ENDCOMP; 
         %end;
         %if &type3 = T %then %do;
            DEFINE p_type3/ order "Type3 P-value" STYLE(COLUMN) = {JUST = C CellWidth=8%}  FORMAT=PVALUE5.3 MISSING; 
            COMPUTE p_type3; 
              IF p_type3 <0.05 THEN CALL DEFINE("p_type3", "STYLE", "STYLE=[FONT_WEIGHT=BOLD]"); 
            ENDCOMP; 
         %end;
         %if &logrank = T %then %do;
            DEFINE logrankp/ order "Log-rank P-value" STYLE(COLUMN) = {JUST = C CellWidth=8%}  FORMAT=PVALUE5.3 MISSING;
            COMPUTE logrankp; 
               IF logrankp <0.05 THEN CALL DEFINE("logrankp", "STYLE", "STYLE=[FONT_WEIGHT=BOLD]"); 
            ENDCOMP;  
         %end;

         COMPUTE ProbChiSq; 
            IF ProbChiSq <0.05 THEN CALL DEFINE("ProbChiSq", "STYLE", "STYLE=[FONT_WEIGHT=BOLD]"); 
         ENDCOMP; 

         compute after covariate; line ''; endcomp;               

         compute after _page_;
            /* If Firth is used */
            %if &firth = T %then %do;
               line @0 "Firths penalized maximum likelihood estimation was used.";
               %if &pl = T %then %do;
               line @0 'HR confidence intervals and p-values are based on the profile penalized log likelihood method.';
               %end;
            %end;
         endcomp;
      RUN;

   %if &doc = T %then %do;
      ods rtf close;
   %end;

   /* Reload original options that were in use before running the macro */
   /* This is mainly to reset the orientation */
   PROC OPTLOAD data=_options;
   RUN;

   /* If not in debug mode then delete temporary data sets */
   %if &debug = F %then %do;
      /* If there are work data sets that should not be deleted */
      %if %sysevalf(%superq(work_sets)~=,boolean) %then %do;
         /* DELETE ALL TEMPORARY DATASETS that were created */
         proc datasets lib=work memtype=data;  
            save &work_sets;
          quit;  
      %end;
      %else %do;
         proc datasets lib=work kill memtype=data;  
         quit; 
      %end;
   %end;

%MEND; 

